/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.commons.lang.ToStringHelper;
import com.squeakysand.jsp.AttributeScope;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspContext;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Extension of the {@link SimpleTagSupport} class, adding useful common methods.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
public abstract class EnhancedSimpleTagSupport extends SimpleTagSupport {

    /**
     * Enumeration of different runtime attribute combinations for validating that the attributes are being passed correctly to the Tag handler at runtime,
     * which is not covered by the JSP Specification, which only provides for JSP compilation time checking of attributes based on the value of the
     * <code>/taglib/tag/attribute/required</code> element in the TLD file.
     */
    protected enum AttributeCombination {
        /**
         * Zero or one of the specified attributes may have a non-null value at runtime.
         */
        AT_MOST_ONE,

        /**
         * At least one of the specified attributes must have a non-null value at runtime.
         */
        AT_LEAST_ONE,

        /**
         * One, and only one of the specified attributes must have a non-null value at runtime.
         */
        EXACTLY_ONE,

        /**
         * All of the specified attributes must have a non-null value at runtime.
         */
        ALL,

        /**
         * Either all of the attributes, or none of the attributes must have a non-null value at runtime.
         */
        ALL_OR_NONE;
    }

    private static final Logger LOG = LoggerFactory.getLogger(EnhancedSimpleTagSupport.class);

    /**
     * Equivalent to calling {@link SimpleTagSupport#getJspContext getJspContext()}.{@link JspContext#findAttribute(String) findAttribute(name)}.
     * 
     * @param name the name of the attribute to search for
     * @return the value associated or null
     */
    protected Object findAttribute(String name) {
        return getJspContext().findAttribute(name);
    }

    /**
     * Equivalent to calling {@link JspContext#findAttribute(String)}, but with generic type checking.
     * 
     * @param name the name of the attribute to search for.
     * @param expectedAttributeType the type of attribute that is being searched for.
     * @return the value associated with the attribute name, or <code>null</code> if no value is found, or a value is found that is of a type that is not
     *         assignable to the <code>expectedAttributeType</code> type (a warning is logged if this situation occurs).
     */
    protected <T> T findAttribute(String name, Class<T> expectedAttributeType) {
        return findAttribute(name, null, expectedAttributeType);
    }

    /**
     * Equivalent to calling {@link JspContext#findAttribute(String)}, but with the addition of default value semantics.
     * 
     * @param name the name of the attribute to search for.
     * @param defaultValue the value to use if no value is found.
     * @return the value associated with the attribute, or the <code>defaultValue</code> if none is found.
     */
    protected Object findAttribute(String name, Object defaultValue) {
        Object result = findAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * Equivalent to calling {@link JspContext#findAttribute(String)}, but with the addition of default value semantics and generic type checking.
     * 
     * @param name the name of the attribute to search for.
     * @param defaultValue the value to use if no value is found, must be assignable to the <code>expectedAttributeType</code> type.
     * @param expectedAttributeType the type of attribute that is being searched for.
     * @return the value associated with the attribute name, or the <code>defaultValue</code> if no value is found, or a value that is of a type that is not
     *         assignable to the <code>expectedAttributeType</code> type (a warning is logged if this situation occurs).
     */
    @SuppressWarnings("unchecked")
    protected <T> T findAttribute(String name, T defaultValue, Class<T> expectedAttributeType) {
        T result = null;
        Object rawAttributeValue = null;
        try {
            rawAttributeValue = findAttribute(name);
            if (rawAttributeValue != null) {
                result = (T) rawAttributeValue;
            }
        } catch (ClassCastException e) {
            LOG.warn(String.format("attribute '%s' was expected to be of type %s, but was of type %s instead", name,
                            ToStringHelper.toString(expectedAttributeType), ToStringHelper.toString(rawAttributeValue.getClass())));
        }
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * Equivalent to calling {@link JspContext#getAttribute(String, int) getJspContext().getAttribute(name, PageContext.APPLICATION_SCOPE)}.
     * 
     * @param name the name of the attribute to get.
     * @return the object associated with the name in the application scope or <code>null</code> if not found.
     */
    protected Object getApplicationAttribute(String name) {
        return getAttribute(name, AttributeScope.APPLICATION);
    }

    /**
     * Equivalent to calling {@link JspContext#getAttribute(String, int) getJspContext().getAttribute(name, PageContext.APPLICATION_SCOPE)}, but with the
     * addition of generic type checking.
     * 
     * @param name the name of the attribute to get.
     * @param expectedAttributeType the expected type of the attribute.
     * @return the value associated with the attribute name, or <code>null</code> if no value is found, or a value is found that is of a type that is not
     *         assignable to the <code>expectedAttributeType</code> type (a warning is logged if this situation occurs).
     */
    protected <T> T getApplicationAttribute(String name, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.APPLICATION, expectedAttributeType);
    }

    /**
     * Equivalent to calling {@link JspContext#getAttribute(String, int) getJspContext().getAttribute(name, PageContext.APPLICATION_SCOPE)}, but with the
     * addition of default value semantics.
     * 
     * @param name the name of the attribute to get.
     * @param defaultValue the value to use if no value is found.
     */
    protected Object getApplicationAttribute(String name, Object defaultValue) {
        return getAttribute(name, AttributeScope.APPLICATION, defaultValue);
    }

    /**
     * Equivalent to calling {@link JspContext#getAttribute(String, int) getJspContext().getAttribute(name, PageContext.APPLICATION_SCOPE)}, but with the
     * addition of default value semantics and generic type checking.
     * 
     * @param name the name of the attribute to get.
     * @param defaultValue the value to use if no value is found.
     * @param expectedAttributeType the expected type of the attribute.
     * @return the value associated with the attribute name, or the <code>defaultValue</code> if no value is found, or a value that is of a type that is not
     *         assignable to the <code>expectedAttributeType</code> type (a warning is logged if this situation occurs).
     */
    protected <T> T getApplicationAttribute(String name, T defaultValue, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.APPLICATION, defaultValue, expectedAttributeType);
    }

    /**
     * Equivalent to calling {@link JspContext#getAttribute#getAttribute(String) getJspContext().getAttribute(name)}.
     * 
     * @param name the name of the attribute to get.
     * @return the value associated with the attribute, which can be <code>null</code>.
     */
    protected Object getAttribute(String name) {
        return getAttribute(name, (Object) null);
    }

    /**
     * 
     * 
     * @param name
     * @param scope
     * @return
     */
    protected Object getAttribute(String name, AttributeScope scope) {
        return getAttribute(name, scope, (Object) null);
    }

    /**
     * 
     * @param name
     * @param scope
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getAttribute(String name, AttributeScope scope, Class<T> expectedAttributeType) {
        return getAttribute(name, scope, null, expectedAttributeType);
    }

    /**
     * 
     * @param name
     * @param scope
     * @param defaultValue
     * @return
     */
    protected Object getAttribute(String name, AttributeScope scope, Object defaultValue) {
        Object result = null;
        if (scope == null) {
            result = getJspContext().getAttribute(name);
        } else {
            result = getJspContext().getAttribute(name, scope.getScopeId());
        }
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * 
     * @param name
     * @param scope
     * @param defaultValue
     * @param expectedAttributeType
     * @return
     */
    @SuppressWarnings("unchecked")
    protected <T> T getAttribute(String name, AttributeScope scope, T defaultValue, Class<T> expectedAttributeType) {
        T result = null;
        Object rawAttributeValue = null;
        try {
            rawAttributeValue = getAttribute(name, scope);
            if (rawAttributeValue != null) {
                result = (T) rawAttributeValue;
            }
        } catch (ClassCastException e) {
            LOG.warn(String.format("attribute '%s' in scope %s was expected to be of type %s, but was of type %s instead", name, scope,
                            ToStringHelper.toString(expectedAttributeType), ToStringHelper.toString(rawAttributeValue.getClass())));
        }
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * 
     * @param name
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getAttribute(String name, Class<T> expectedAttributeType) {
        return getAttribute(name, (T) null, expectedAttributeType);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @return
     */
    protected Object getAttribute(String name, Object defaultValue) {
        Object result = getJspContext().getAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @param expectedAttributeType
     * @return
     */
    @SuppressWarnings("unchecked")
    protected <T> T getAttribute(String name, T defaultValue, Class<T> expectedAttributeType) {
        T result = null;
        Object rawAttributeValue = null;
        try {
            rawAttributeValue = getAttribute(name);
            if (rawAttributeValue != null) {
                result = (T) rawAttributeValue;
            }
        } catch (ClassCastException e) {
            LOG.warn(String.format("attribute '%s' was expected to be of type %s, but was of type %s instead", name,
                            ToStringHelper.toString(expectedAttributeType), ToStringHelper.toString(rawAttributeValue.getClass())));
        }
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * 
     * @return
     */
    protected JspWriter getJspWriter() {
        return getJspContext().getOut();
    }

    /**
     * 
     * @param name
     * @return
     */
    protected Object getPageAttribute(String name) {
        return getAttribute(name, AttributeScope.PAGE);
    }

    /**
     * 
     * @param name
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getPageAttribute(String name, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.PAGE, expectedAttributeType);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @return
     */
    protected Object getPageAttribute(String name, Object defaultValue) {
        return getAttribute(name, AttributeScope.PAGE, defaultValue);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getPageAttribute(String name, T defaultValue, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.PAGE, defaultValue, expectedAttributeType);
    }

    /**
     * 
     * @return
     */
    protected PageContext getPageContext() {
        return (PageContext) getJspContext();
    }

    /**
     * 
     * @return
     */
    protected ServletRequest getRequest() {
        return getPageContext().getRequest();
    }

    /**
     * 
     * @param name
     * @return
     */
    protected Object getRequestAttribute(String name) {
        return getAttribute(name, AttributeScope.REQUEST);
    }

    /**
     * 
     * @param name
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getRequestAttribute(String name, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.REQUEST, expectedAttributeType);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @return
     */
    protected Object getRequestAttribute(String name, Object defaultValue) {
        return getAttribute(name, AttributeScope.REQUEST, defaultValue);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getRequestAttribute(String name, T defaultValue, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.REQUEST, defaultValue, expectedAttributeType);
    }

    /**
     * 
     * @param parameterName
     * @return
     */
    protected String getRequestParameter(String parameterName) {
        return getRequest().getParameter(parameterName);
    }

    /**
     * 
     * @param parameterName
     * @return
     */
    protected List<String> getRequestParameterValues(String parameterName) {
        List<String> result = null;
        String[] parameterValues = getRequest().getParameterValues(parameterName);
        if (parameterValues == null || parameterValues.length < 1) {
            result = Collections.emptyList();
        } else {
            result = Arrays.asList(parameterValues);
        }
        return result;
    }

    /**
     * 
     * @return
     */
    protected ServletResponse getResponse() {
        return getPageContext().getResponse();
    }

    /**
     * 
     * @return
     */
    protected ServletConfig getServletConfig() {
        return getPageContext().getServletConfig();
    }

    /**
     * 
     * @param name
     * @return
     */
    protected Object getServletConfigInitParameter(String name) {
        return getPageContext().getServletConfig().getInitParameter(name);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @return
     */
    protected Object getServletConfigInitParameter(String name, Object defaultValue) {
        Object result = getServletConfigInitParameter(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * 
     * @return
     */
    protected ServletContext getServletContext() {
        return getPageContext().getServletContext();
    }

    /**
     * 
     * @return
     */
    protected HttpSession getSession() {
        return getPageContext().getSession();
    }

    /**
     * 
     * @param name
     * @return
     */
    protected Object getSessionAttribute(String name) {
        return getAttribute(name, AttributeScope.SESSION);
    }

    /**
     * 
     * @param name
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getSessionAttribute(String name, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.SESSION, expectedAttributeType);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @return
     */
    protected Object getSessionAttribute(String name, Object defaultValue) {
        return getAttribute(name, AttributeScope.SESSION, defaultValue);
    }

    /**
     * 
     * @param name
     * @param defaultValue
     * @param expectedAttributeType
     * @return
     */
    protected <T> T getSessionAttribute(String name, T defaultValue, Class<T> expectedAttributeType) {
        return getAttribute(name, AttributeScope.SESSION, defaultValue, expectedAttributeType);
    }

    /**
     * 
     * @param name
     */
    protected void removeApplicationAttribute(String name) {
        getJspContext().removeAttribute(name, AttributeScope.APPLICATION.getScopeId());
    }

    /**
     * 
     * @param name
     */
    protected void removePageAttribute(String name) {
        getJspContext().removeAttribute(name, AttributeScope.PAGE.getScopeId());
    }

    /**
     * 
     * @param name
     */
    protected void removeRequestAttribute(String name) {
        getJspContext().removeAttribute(name, AttributeScope.REQUEST.getScopeId());
    }

    /**
     * 
     * @param name
     */
    protected void removeSessionAttribute(String name) {
        getJspContext().removeAttribute(name, AttributeScope.SESSION.getScopeId());
    }

    /**
     * 
     * @param name
     * @param value
     */
    protected void setApplicationAttribute(String name, Object value) {
        setAttribute(name, value, AttributeScope.APPLICATION);
    }

    /**
     * 
     * @param name
     * @param value
     */
    protected void setAttribute(String name, Object value) {
        setAttribute(name, value, null);
    }

    /**
     * 
     * @param name
     * @param value
     * @param scope
     */
    protected void setAttribute(String name, Object value, AttributeScope scope) {
        if (scope == null) {
            getJspContext().setAttribute(name, value);
        } else {
            getJspContext().setAttribute(name, value, scope.getScopeId());
        }
    }

    /**
     * 
     * @param name
     * @param value
     */
    protected void setPageAttribute(String name, Object value) {
        setAttribute(name, value, AttributeScope.PAGE);
    }

    /**
     * 
     * @param name
     * @param value
     */
    protected void setRequestAttribute(String name, Object value) {
        setAttribute(name, value, AttributeScope.REQUEST);
    }

    /**
     * 
     * @param name
     * @param value
     */
    protected void setSessionAttribute(String name, Object value) {
        setAttribute(name, value, AttributeScope.SESSION);
    }

    /**
     * 
     * @param attribute1name
     * @param attribute1value
     * @param attribute2name
     * @param attribute2value
     * @param combination
     * @throws JspException
     */
    protected void validateAttributeCombination(String attribute1name, Object attribute1value, String attribute2name, Object attribute2value,
                    AttributeCombination combination) throws JspException {
        if (StringUtils.isNullOrEmpty(attribute1name)) {
            throw new JspException("attribute1name parameter cannot be null or empty");
        }
        if (StringUtils.isNullOrEmpty(attribute2name)) {
            throw new JspException("attribute2name parameter cannot be null or empty");
        }
        if (combination == null) {
            throw new JspException("combination parameter cannot be null");
        }
        boolean attribute1isNotNull = attribute1value != null;
        boolean attribute2isNotNull = attribute2value != null;
        switch (combination) {
        case AT_MOST_ONE:
            if (attribute1isNotNull && attribute2isNotNull) {
                throw new JspException(String.format("only 1 of %s or %s attributes can have a runtime value set", attribute1name, attribute2name));
            }
            break;
        case AT_LEAST_ONE:
            if (!attribute1isNotNull && !attribute2isNotNull) {
                throw new JspException(String.format("at least 1 of %s or %s attributes must have a runtime value set", attribute1name, attribute2name));
            }
            break;
        case EXACTLY_ONE:
            if (!attribute1isNotNull && !attribute2isNotNull) {
                throw new JspException(String.format("one attribute out of %s or %s must have a runtime value set", attribute1name, attribute2name));
            } else if (attribute1isNotNull && attribute2isNotNull) {
                throw new JspException(String.format("only one attribute of %s or %s can have a runtime value set", attribute1name, attribute2name));
            }
            break;
        case ALL:
            if (!attribute1isNotNull || !attribute2isNotNull) {
                throw new JspException(String.format("both attributes {} and {} must have runtime values set", attribute1name, attribute2name));
            }
            break;
        case ALL_OR_NONE:
            if (attribute1isNotNull != attribute2isNotNull) {
                throw new JspException(String.format("if either {} or {} has a runtime value set, then the other must also have a runtime value set",
                                attribute1name, attribute2name));
            }
            break;
        default:
            throw new JspException(String.format("unknown AttributeCombination value - %s", combination.name()));
        }
    }

    /**
     * 
     * @param attributeName
     * @param attributeValue
     * @throws JspException
     */
    protected void validateNotNullAttribute(String attributeName, Object attributeValue) throws JspException {
        if (attributeValue == null) {
            throw new JspException(String.format("attribute %s must have a non-null runtime value", attributeName));
        }
    }

    /**
     * 
     * @param attributeName
     * @param attributeValue
     * @throws JspException
     */
    protected void validateNotNullOrEmptyAttribute(String attributeName, String attributeValue) throws JspException {
        if (StringUtils.isNullOrEmpty(attributeName)) {
            throw new JspException(String.format("String attribute %s must have a non-null and non-empty runtime value", attributeName));
        }
    }

}
