/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext;

import com.squeakysand.commons.io.FileFinder;
import com.squeakysand.commons.lang.SystemProperties;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.filefilter.IOFileFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Helper methods for working with JSP Tag Libraries. These classes are for working with the Tag Library itself and are not intended to be used for the actual
 * implementation of Tag handler classes within a Tag Library.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
public class TagLibraryUtils {

    private static final Logger LOG = LoggerFactory.getLogger(TagLibraryUtils.class);

    /**
     * Searches for a TEI class.
     * 
     * @param klass the Tag handler Class.
     * @param loader the class loader to search in.
     * @return the TEI class or <code>null</code> if one was not found.
     */
    public static Class<?> findTeiClass(Class<?> klass, ClassLoader loader) {
        LOG.trace("findTeiClass called: klass = {}, loader = {}", klass, loader);
        Class<?> teiClass = null;
        String teiClassName = generateDefaultTeiClassName(klass.getName());
        try {
            teiClass = loader.loadClass(teiClassName);
        } catch (ClassNotFoundException ex) {
            LOG.debug("unable to find TEI class with name {}", teiClassName);
        }
        return teiClass;
    }

    /**
     * Searches for a TEI class.
     * 
     * @param qualifiedTagClassName the name of the Tag handler class.
     * @param srcDir the root of the Java source tree to search in.
     * @return the qualified name of the TEI class, or <code>null</code> if none was found.
     * @throws IOException if there is a problem reading from the <code>srcDir</code> directory.
     */
    public static String findTeiClass(String qualifiedTagClassName, final File srcDir) throws IOException {
        LOG.trace("findTeiClass called: qualifiedTagClassName = {}, srcDir = {}", qualifiedTagClassName, srcDir);
        String result = null;
        String defaultQualifiedTeiClassName = generateDefaultTeiClassName(qualifiedTagClassName);
        final String defaultTeiPackageName = defaultQualifiedTeiClassName.substring(0, defaultQualifiedTeiClassName.lastIndexOf('.'));
        final String defaultTeiClassName = defaultQualifiedTeiClassName.substring(defaultQualifiedTeiClassName.lastIndexOf('.') + 1);
        LOG.debug("tei class info: {} {}", defaultQualifiedTeiClassName, defaultTeiClassName);
        IOFileFilter fileFilter = new IOFileFilter() {

            @Override
            public boolean accept(File file) {
                return accept(file.getParentFile(), file.getName());
            }

            @Override
            public boolean accept(File dir, String name) {
                LOG.debug("accept: {}/{}", dir, name);
                boolean result = false;
                if (name.equals(defaultTeiClassName + ".java")) {
                    try {
                        String srcDirPath = srcDir.getCanonicalPath();
                        LOG.debug("srcDirPath: {}", srcDirPath);
                        String dirPath = dir.getCanonicalPath();
                        LOG.debug("dirPath: {}", dirPath);
                        String trimmedPath = dirPath.substring(srcDirPath.length() + 1);
                        LOG.debug("trimmedPath: {}", trimmedPath);
                        String packageName = trimmedPath.replace(SystemProperties.FILE_SEPARATOR, ".");
                        LOG.debug("packageName: {}", packageName);
                        if (defaultTeiPackageName.equals(packageName)) {
                            result = true;
                        }
                    } catch (IOException e) {
                        LOG.error(e.getMessage(), e);
                    }
                }
                LOG.debug("returning {} from accept", result);
                return result;
            }

        };
        FileFinder fileFinder = new FileFinder(fileFilter);
        List<File> files = fileFinder.findFiles(srcDir);
        if (files != null && files.size() > 0) {
            result = defaultQualifiedTeiClassName;
        }
        LOG.trace("returnining Tei class name: {}", result);
        return result;
    }

    /**
     * Generates default names for Tags based on the Tag handler class name.
     * 
     * @param klass the Tag implementation class.
     * @return the default name to use for the Tag in the TLD file.
     * 
     * @see #generateDefaultTagName(String)
     */
    public static String generateDefaultTagName(Class<?> klass) {
        String simpleClassName = klass.getSimpleName();
        return generateDefaultTagName(simpleClassName);
    }

    /**
     * Generates default names for Tags based on the Tag handler class name.
     * 
     * The following steps are applied to the supplied simple class name:
     * <ol>
     * <li>the first letter of the name is changed to lower case</li>
     * <li>if the name ends in the suffix <code>Tag</code>, then it is removed</li>
     * </ol>
     * 
     * For example, if I pass in the simple class name of Tag handler as <code>MyImportantTag<code>, then this method would return <code>myImportant</code>.
     * 
     * @param simpleClassName the name of the Tag handler class, obtained using {@link Class#getSimpleName()}, cannot be <code>null</code>.
     * @return a default name for the Tag handler to use in a TLD file.
     */
    public static String generateDefaultTagName(String simpleClassName) {
        String result = simpleClassName;
        // lowercase the first letter
        result = Character.toLowerCase(result.charAt(0)) + result.substring(1);
        // if the class result ends in "Tag", then crop it off
        if (result.endsWith("Tag")) {
            result = result.substring(0, result.length() - 3);
        }
        return result;
    }

    /**
     * Generates the default TEI class name based on the qualified name of the Tag handler class supplied.
     * 
     * @param qualifiedTagClassName the qualified name of the Tag handler class, cannot be <code>null</code>.
     * @return the qualified name of the Tag handler class with <code>ExtraInfo</code> appended, as per the JSP Specification.
     */
    public static String generateDefaultTeiClassName(String qualifiedTagClassName) {
        return qualifiedTagClassName + "ExtraInfo";
    }

}
