/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext.annotations;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Marks a public static method to be included in a JSP Tag Library.
 * 
 * For example:
 * 
 * <pre>
 * package mypackage;
 * 
 * public class MyElFunctions {
 *     ...
 *     /**
 *      * This is the JavaDoc comment for the getSomeValue method.
 *      *&#47;
 *     &commat;JspElFunction
 *     public static String getSomeValue() {
 *         ...
 *     }
 *     ...
 * }
 * </pre>
 * 
 * would result in the following being added to the TLD:
 * 
 * <pre>
 * &lt;taglib>
 *     ...
 *     &lt;function>
 *         &lt;description>This is the JavaDoc comment for the getSomeValue method.&lt;/description>
 *         &lt;name>getSomeValue&lt;/name>
 *         &lt;function-class>mypackage.MyElFunctions&lt;/function-class>
 *         &lt;function-signature>java.lang.String getSomeValue()&lt;/function-signature>
 *     &lt;/function>
 *     ...
 * &lt;/taglib>
 * </pre>
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface JspElFunction {

    /**
     * An example of using the function, which will be used in the &lt;example> element of the TLD.
     * 
     * There is no default value.
     */
    String example() default "";

    /**
     * Override the default function name.
     * 
     * Default value is the name of the method.
     */
    String name() default "";

}
