/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext.annotations;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.servlet.jsp.tagext.JspFragment;

/**
 * Adds an <code>&lt;attribute></code> sub-element to a <code>&lt;tag></code> element in the TLD when applied to a
 * method that is a JavaBean setter.
 * 
 * For example:
 * 
 * <pre>
 *  ...
 *  /**
 *   * This is the method level JavaDoc comment.
 *   *&#47;
 *  &commat;JspTagAttribute(required = true)
 *  public void setMyAttribute(String value) {
 *      ...
 *  }
 *  ...
 * </pre>
 * 
 * would result in the following being added to the TLD:
 * 
 * <pre>
 * &lt;taglib>
 *     ...
 *     &lt;tag>
 *        ...
 *        &lt;attribute>
 *            &lt;description>This is the method level JavaDoc comment.&lt;/description>
 *               &lt;name>myAttribute&lt;/name>
 *               &lt;required>true&lt;/required>
 *            &lt;/description>
 *        &lt;/attribute>
 *        ...
 *     &lt;/tag>
 *     ...
 * &lt;/taglib>
 * </pre>
 * 
 * If the type of the parameter to the JavaBean setter method is anything other than {@link String}, then the
 * appropriate <code>&lt;type></code> element with the fully qualified class name as the value will also be added to the
 * TLD.
 * 
 * If the type of the parameter to the JavaBean setter method happens to be {@link JspFragment}, then the
 * <code>&lt;fragment>true&lt;/fragment></code> element will be added to the TLD.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface JspTagAttribute {

    /**
     * Value for the <code>&lt;required></code> element in the TLD for this attribute.
     * 
     * The default value is <code>false</code> (from the specification).
     */
    boolean required() default false;

    /**
     * Value for the <code>&lt;rtexprvalue></code> element in the TLD for this attribute.
     * 
     * The default value is <code>false</code> (from the specification).
     */
    boolean rtexprvalue() default false;

}
