/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext.annotations;

import com.squeakysand.jsp.tagext.VariableScope;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;

/**
 * Specialized version of the {@link JspTagVariable} annotation for use when the tag class creates a variable that is a
 * sub-type of {@link Collection} and allows the author to formally document the element type in the collection, which
 * is not currently supported by the Taglib Descriptor specification.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.FIELD, ElementType.METHOD})
public @interface JspTagCollectionVariable {

    /**
     * The value for the <code>&lt;declare></code> element.
     * 
     * The default value is <code>false</code> (from the specification).
     */
    boolean declare() default false;

    /**
     * The type of the elements in the {@link Collection}. There is no where officially in a TLD to define this, so
     * additional text is added to the <code>&lt;description></code> tag for documentation purposes.
     * 
     * The default value is {@link String}.
     */
    Class<?> elementClass() default String.class;

    /**
     * The value for the <code>&lt;scope></code> element.
     * 
     * The default value is <code>NESTED</code> (from the specification).
     */
    VariableScope scope() default VariableScope.NESTED;

    /**
     * The type of the variable that will be injected into the JSP page context. Must be assignable to
     * {@link Collection}. In the TLD, this is indicated by the <code>&lt;variable-class></code> element.
     * 
     * The default value is {@link Collection} - developers may choose to specify a sub-type of the {@link Collection}
     * interface here (e.g. {@link Set} or {@link ArrayList}), however for most uses of a variable within a JSP page
     * this is not necessary.
     */
    Class<? extends Collection> variableClass() default Collection.class;

}
